﻿using Newtonsoft.Json;
using Oracle.DataAccess.Client;
using Oracle.DataAccess.Types;
using System;
using System.Configuration;
using System.Data;
using System.Web.Services;
using System.Xml;

namespace ercapis
{
    /// <summary>
    /// Retrieves Council Tax Information from Orbis 
    /// Based on Northgate Stored Procedures contained in 'ORBiS CTAX API_XML Functions v3'
    /// </summary>
    [WebService(Namespace = "http://tempuri.org/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [System.ComponentModel.ToolboxItem(false)]
    // To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
    // [System.Web.Script.Services.ScriptService]
    public class CouncilTax : System.Web.Services.WebService
    {


        #region Properties (Request Headers)

        /// <summary>
        /// Returns the Application ID
        /// </summary>
        private string getApplicationID
        {
            get
            {
                return "ORBIS CTAX API";
            }

        }

        /// <summary>
        /// Returns an Oracle friendly timestamp
        /// </summary>
        private string getOracleTimeStamp
        {
            get
            {
                string date = DateTime.Now.ToString("yyyy-MM-dd");
                string time = DateTime.Now.ToString("HH:mm:ss");
                string timestamp = date + "T" + time;

                return timestamp;
            }

        }

        /// <summary>
        /// Return a unique(ish) request id.  Returns the current timestamp and a random number
        /// </summary>
        private string getRequestID
        {
            get
            {
                
                // Random rnd = new Random();
                // int rndnumber = rnd.Next();
                return this.getOracleTimeStamp + "::" + Guid.NewGuid().ToString("n").Substring(0, 20);

            }

        }

        /// <summary>
        /// Returns the Application Type
        /// </summary>
        private string getType
        {
            get
            {
                return "CTAX";
            }

        }

        #endregion




        #region XML

        /// <summary>
        /// This API will return a list of discount types that are marked as being allowed to be used by the web (for either ‘Request Only’ or ‘Creation Allowed’)
        /// </summary>
        /// <returns></returns>
        [WebMethod(Description = "Will return a list of discount types that are marked as being allowed to be used by the web (for either ‘Request Only’ or ‘Creation Allowed’)")]
        public string getAvailableDiscTypes()
        {

            string xmlResponse = string.Empty;
            string xmlRequest  = string.Empty;

            try
            {
                
                xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                xmlRequest += "<header>";
                xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                xmlRequest += "	<type>" + this.getType + "</type>";
                xmlRequest += "</header>";
                xmlRequest += "<request>";
                xmlRequest += "<getavailabledisctypes>";
                xmlRequest += "</getavailabledisctypes>";
                xmlRequest += "</request>";
                xmlRequest += "</orbisrequest>";

                xmlResponse = orbisSend(xmlRequest);

            }
            catch (Exception ex)
            {
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }


            return xmlResponse;


        }


        /// <summary>
        /// This API will retrieve detailed information for a specific ctax account.
        /// </summary>
        /// <param name="accountID"></param>
        /// <returns></returns>
        [WebMethod(Description = "Will get all transactions that have been made to/from an account")]
        public string accountDetailsRequest(string accountID)
        {

            string xmlResponse = string.Empty;
            string xmlRequest  = string.Empty;

            try
            {

                xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                xmlRequest += "<header>";
                xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                xmlRequest += "	<type>" + this.getType + "</type>";
                xmlRequest += "	<reference>" + accountID + "</reference>";
                xmlRequest += "</header>";
                xmlRequest += "<request>";
                xmlRequest += "<accountdetailsrequest>";
                xmlRequest += "</accountdetailsrequest>";
                xmlRequest += "</request>";
                xmlRequest += "</orbisrequest>";

                xmlResponse = orbisSend(xmlRequest);

            }
            catch (Exception ex)
            {
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }


            return xmlResponse;
        }


        /// <summary>
        /// This API will return information for a specific PIN. The PIN will have been identified on another API.
        /// </summary>
        /// <param name="pin"></param>
        /// <returns></returns>
        [WebMethod(Description = "Will return information for a PIN that is passed in")]
        public string personAccountRequest(string pin)
        {

            string xmlResponse = string.Empty;
            string xmlRequest  = string.Empty;

            try
            {

                xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                xmlRequest += "<header>";
                xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                xmlRequest += "	<type>" + this.getType + "</type>";
                xmlRequest += "	<perpin>" + pin + "</perpin>";
                xmlRequest += "</header>";
                xmlRequest += "<request>";
                xmlRequest += "<personaccountrequest>";
                xmlRequest += "</personaccountrequest>";
                xmlRequest += "</request>";
                xmlRequest += "</orbisrequest>";

                xmlResponse = orbisSend(xmlRequest);

            }
            catch (Exception ex)
            {
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }


            return xmlResponse;


        }


        /// <summary>
        /// This API will return all the information for a PIN based on an account id that has been passed in. The process will use that account id to get a list of PINs and return all information for this list of PINS.
        /// </summary>
        [WebMethod(Description = "Will return all the information for a PIN based on an account id that has been passed in. The process will use that account id to get a list of PINs and return all information for this list of PINS")]
        public string personInfoRequest(string accountID, string forename, string surname, string street, string houseNumber, string houseName, string postcode)
        {

            string xmlResponse = string.Empty;
            string xmlRequest  = string.Empty;

            try
            {

                xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                xmlRequest += "<header>";
                xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                xmlRequest += "	<type>" + this.getType + "</type>";
                xmlRequest += "	<accountno>" + accountID + "</accountno>";
                xmlRequest += "	<forename>" + forename + "</forename>";
                xmlRequest += "	<surname>" + surname + "</surname>";
                xmlRequest += "	<street>" + street + "</street>";
                xmlRequest += "	<houseno>" + houseNumber + "</houseno>";
                xmlRequest += "	<housename>" + houseName + "</housename>";
                xmlRequest += "	<postcode>" + postcode + "</postcode>";
                xmlRequest += "</header>";
                xmlRequest += "<request>";
                xmlRequest += "<personinforequest>";
                xmlRequest += "</personinforequest>";
                xmlRequest += "</request>";
                xmlRequest += "</orbisrequest>";

                xmlResponse = orbisSend(xmlRequest);

            }
            catch (Exception ex)
            {
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }


            return xmlResponse;


        }









        /// <summary>
        /// This API will return the property address for a given a/c number (or UPRN) plus all the liable people that are associated with the property
        /// </summary>
        /// <param name="accountID"></param>
        /// <param name="UPRN"></param>
        /// <returns></returns>
        [WebMethod(Description = "Will return the property address plus all the liable people that are associated with the property")]
        public string propertyEnquiry(string accountID)
        {

            string xmlResponse = string.Empty;
            string xmlRequest =  string.Empty;

            try
            {

                xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                xmlRequest += "<header>";
                xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                xmlRequest += "	<type>" + this.getType + "</type>";
                xmlRequest += "	<reference>" + accountID + "</reference>";
                //  xmlRequest += "	<UPRN>" + UPRN + "</UPRN>";  // Passing only UPRN results in an error 'Error Message : Account Id() is invalid'.
                xmlRequest += "</header>";
                xmlRequest += "<request>";
                xmlRequest += "<propertyenquiry>";
                xmlRequest += "</propertyenquiry>";
                xmlRequest += "</request>";
                xmlRequest += "</orbisrequest>";

                xmlResponse = orbisSend(xmlRequest);

            }
            catch (Exception ex)
            {
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }


            return xmlResponse;


        }





        /// <summary>
        /// This API will take an account id, surname and postcode and validate that the surname and postcode input are valid for the account number provided. If it passes the validation then details will be output in relation to the property address, involvement & group.
        /// </summary>
        /// <param name="accountID"></param>
        /// <param name="surname"></param>
        /// <param name="postcode"></param>
        /// <returns></returns>
        [WebMethod(Description = "Will take an account id, surname and postcode and validate that the surname and postcode input are valid for the account number provided")]
        public string authenticatePerson(string accountID, string surname, string postcode)
        {

            string xmlResponse = string.Empty;
            string xmlRequest  = string.Empty;

            try
            {

                xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                xmlRequest += "<header>";
                xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                xmlRequest += "	<type>" + this.getType + "</type>";
                xmlRequest += "	<reference>" + accountID + "</reference>";
                xmlRequest += "	<surname>" + surname + "</surname>";
                xmlRequest += "	<postcode>" + postcode + "</postcode>";
                xmlRequest += "</header>";
                xmlRequest += "<request>";
                xmlRequest += "<authenticateperson>";
                xmlRequest += "</authenticateperson>";
                xmlRequest += "</request>";
                xmlRequest += "</orbisrequest>";

                xmlResponse = orbisSend(xmlRequest);

            }
            catch (Exception ex)
            {
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }


            return xmlResponse;


        }






        /// <summary>
        /// This API will return demand information relating to an account id that is passed in.
        /// </summary>
        /// <param name="accountID"></param>
        /// <returns></returns>
        [WebMethod(Description = "Will return demand information relating to an account id that is passed in")]
        public string accountAllDemands(string accountID)
        {

            string xmlResponse = string.Empty;
            string xmlRequest = string.Empty;

            try
            {

                xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                xmlRequest += "<header>";
                xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                xmlRequest += "	<type>" + this.getType + "</type>";
                xmlRequest += "	<reference>" + accountID + "</reference>";
                xmlRequest += "</header>";
                xmlRequest += "<request>";
                xmlRequest += "<accountalldemands>";
                xmlRequest += "</accountalldemands>";
                xmlRequest += "</request>";
                xmlRequest += "</orbisrequest>";

                xmlResponse = orbisSend(xmlRequest);

            }
            catch (Exception ex)
            {
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }


            return xmlResponse;
        }


        /// <summary>
        /// This API will allow the request for a reprint of a previously issued bill
        /// </summary>
        /// <param name="accountID"></param>
        /// <param name="billType"></param>
        /// <returns></returns>
        [WebMethod(Description = "Will attempt to process a request for a duplicate bill")]
        public string requestReprintBill(string accountID, string billType)
        {

            string xmlResponse = string.Empty;
            string xmlRequest = string.Empty;

            try
            {

                xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                xmlRequest += "<header>";
                xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                xmlRequest += "	<type>" + this.getType + "</type>";
                xmlRequest += "	<reference>" + accountID + "</reference>";
                xmlRequest += "	<requestbilltype>" + billType + "</requestbilltype>";
                xmlRequest += "</header>";
                xmlRequest += "<request>";
                xmlRequest += "<requestreprintbill>";
                xmlRequest += "</requestreprintbill>";
                xmlRequest += "</request>";
                xmlRequest += "</orbisrequest>";

                xmlResponse = orbisSend(xmlRequest);

            }
            catch (Exception ex)
            {
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }


            return xmlResponse;
        }




        /// <summary>
        /// This API will bring back all the available payment methods & frequencies in the system.
        /// </summary>
        /// <returns></returns>
        [WebMethod(Description = "Will return all valid payment methods + valid frequencies")]
        public string getAvailablePayMethods()
        {

            string xmlResponse = string.Empty;
            string xmlRequest = string.Empty;

            try
            {

                xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                xmlRequest += "<header>";
                xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                xmlRequest += "	<type>" + this.getType + "</type>";
                xmlRequest += "</header>";
                xmlRequest += "<request>";
                xmlRequest += "<getavailablepaymethods>";
                xmlRequest += "</getavailablepaymethods>";
                xmlRequest += "</request>";
                xmlRequest += "</orbisrequest>";

                xmlResponse = orbisSend(xmlRequest);

            }
            catch (Exception ex)
            {
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }


            return xmlResponse;
        }





        /// <summary>
        /// This API will return recovery details for an account id that has been passed in.
        /// </summary>
        /// <returns></returns>
        [WebMethod(Description = "Will return recovery details for an account id that has been passed in")]
        public string getRecoveryDetails(string accountID)
        {

            string xmlResponse = string.Empty;
            string xmlRequest = string.Empty;

            try
            {

                xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                xmlRequest += "<header>";
                xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                xmlRequest += "	<type>" + this.getType + "</type>";
                xmlRequest += "	<reference>" + accountID + "</reference>";
                xmlRequest += "</header>";
                xmlRequest += "<request>";
                xmlRequest += "<getrecoverydetails>";
                xmlRequest += "</getrecoverydetails>";
                xmlRequest += "</request>";
                xmlRequest += "</orbisrequest>";

                xmlResponse = orbisSend(xmlRequest);

            }
            catch (Exception ex)
            {
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }


            return xmlResponse;
        }








        /// <summary>
        /// This API will take in an account id or an involvement id and return all memos for the account or for the associated involvement (ordered by memo create date desc).
        /// </summary>
        /// <returns></returns>
        [WebMethod(Description = "Will return all memos for the associated involvement.")]
        public string accountMemos(string accountID)
        {

            string xmlResponse = string.Empty;
            string xmlRequest = string.Empty;

            try
            {

                xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                xmlRequest += "<header>";
                xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                xmlRequest += "	<type>" + this.getType + "</type>";
                // xmlRequest += "	<uprn>" + UPRN + "</uprn>"; // Response is empty when checking only by this, no error, just empty xml
                xmlRequest += "	<reference>" + accountID + "</reference>";
                // xmlRequest += "	<involvement>" + involvement + "</involvement>"; // Involvement ID was bland when tested against an account id: '<memoInvolvementId/>'
                xmlRequest += "</header>";
                xmlRequest += "<request>";
                xmlRequest += "<accountmemos>";
                xmlRequest += "</accountmemos>";
                xmlRequest += "</request>";
                xmlRequest += "</orbisrequest>";

                xmlResponse = orbisSend(xmlRequest);

            }
            catch (Exception ex)
            {
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }


            return xmlResponse;
        }



















        /// <summary>
        /// This API will return a list of exemption types that are marked as being allowed to be used by the web (for either ‘Request Only’ or ‘Creation Allowed’)
        /// </summary>
        /// <returns></returns>
        [WebMethod(Description = "Will return a list of exemption types that are marked as being allowed to be used by the web (for either ‘Request Only’ or ‘Creation Allowed’)")]
        public string getAvailableExemptTypes()
        {

            string xmlResponse = string.Empty;
            string xmlRequest = string.Empty;

            try
            {

                xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                xmlRequest += "<header>";
                xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                xmlRequest += "	<type>" + this.getType + "</type>";
                xmlRequest += "</header>";
                xmlRequest += "<request>";
                xmlRequest += "<getavailableexempttypes>";
                xmlRequest += "</getavailableexempttypes>";
                xmlRequest += "</request>";
                xmlRequest += "</orbisrequest>";

                xmlResponse = orbisSend(xmlRequest);

            }
            catch (Exception ex)
            {
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }


            return xmlResponse;
        }












        /// <summary>
        /// This API will take in user payment method, frequency, pay day, method type and if DD – sortcode, bank a/c number, bank a/c name, PIN of associated person plus the BACS required contact details. Will then validate these details and change the payment method
        /// </summary>
        /// <returns></returns>
        [WebMethod(Description = "Will take in user payment method, frequency, pay day, method type and if DD – sortcode, bank a/c number, bank a/c name, PIN of associated person plus the BACS required contact details. Will then validate these details and change the payment method.")]
        public string changePaymentMethod(string accountID, string pin, string paymentMethod, string paymentFrequency, string ddPaymentSortCode, string ddBankAccountNo, string ddBankAccountName, string ddBacsSourceDetails, string ddBacsSourcePhoneNo)
        {

            string xmlResponse = string.Empty;
            string xmlRequest = string.Empty;

            try
            {

                xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                xmlRequest += "<header>";
                xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                xmlRequest += "	<type>" + this.getType + "</type>";
                xmlRequest += "	<reference>" + accountID + "</reference>";
                xmlRequest += "	<perpin>" + pin + "</perpin>";
                xmlRequest += "	<paymentmethod>" + paymentMethod + "</paymentmethod>";
                xmlRequest += "	<paymentfrequency>" + paymentFrequency + "</paymentfrequency>";


                // If Direct Debit is the new payment method
                xmlRequest += "	<paymentsortcode>" + ddPaymentSortCode + "</paymentsortcode>";
                xmlRequest += "	<bankaccountno>" + ddBankAccountNo + "</bankaccountno>";
                xmlRequest += "	<bankaccountname>" + ddBankAccountName + "</bankaccountname>";
                xmlRequest += "	<bacssourcedetails>" + ddBacsSourceDetails + "</bacssourcedetails>";
                xmlRequest += "	<bacssourcephoneno>" + ddBacsSourcePhoneNo + "</bacssourcephoneno>";


                xmlRequest += "</header>";
                xmlRequest += "<request>";
                xmlRequest += "<changepaymentmethod>";
                xmlRequest += "</changepaymentmethod>";
                xmlRequest += "</request>";
                xmlRequest += "</orbisrequest>";

                xmlResponse = orbisSend(xmlRequest);

            }
            catch (Exception ex)
            {
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }


            return xmlResponse;
        }






        /// <summary>
        /// This API will take in an account id and an address and return a code of success or failure as to whether the correspondence address for the group related to that account id was updated.
        /// </summary>
        /// <returns></returns>
        [WebMethod(Description = "Will take in an address and return a code of success or failure")]
        public string changeCorrespondenceAddress(string accountID, string pin, string paon, string saon, string street, string locality, string town, string posttown, string adminarea, string postcode)
        {

            string xmlResponse = string.Empty;
            string xmlRequest = string.Empty;

            try
            {

                xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                xmlRequest += "<header>";
                xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                xmlRequest += "	<type>" + this.getType + "</type>";
                xmlRequest += "	<reference>" + accountID + "</reference>";
                xmlRequest += "	<perpin>" + pin + "</perpin>";
                xmlRequest += "	<paon>" + paon + "</paon>";
                xmlRequest += "	<saon>" + saon + "</saon>";
                xmlRequest += "	<street>" + street + "</street>";
                xmlRequest += "	<locality>" + locality + "</locality>";
                xmlRequest += "	<town>" + town + "</town>";
                xmlRequest += "	<posttown>" + posttown + "</posttown>";
                xmlRequest += "	<adminarea>" + adminarea + "</adminarea>";
                xmlRequest += "	<postcode>" + postcode + "</postcode>";



                xmlRequest += "</header>";
                xmlRequest += "<request>";
                xmlRequest += "<changecorrespondenceaddress>";
                xmlRequest += "</changecorrespondenceaddress>";
                xmlRequest += "</request>";
                xmlRequest += "</orbisrequest>";

                xmlResponse = orbisSend(xmlRequest);

            }
            catch (Exception ex)
            {
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }


            return xmlResponse;
        }






        /// <summary>
        /// This API will allow the user to input a new memo against an account id and return a code of success or failure as to whether the memo was added to the account id or not.
        /// </summary>
        /// <returns></returns>
        [WebMethod(Description = "Will create a new memo against the account based on the detail presented")]
        public string createNewMemo(string username, string accountID, string uprn, string memoText, string memoGroup)
        {

            string xmlResponse = string.Empty;
            string xmlRequest = string.Empty;

            try
            {

                xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                xmlRequest += "<header>";
                xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                xmlRequest += "	<type>" + this.getType + "</type>";
                xmlRequest += "	<username>" + username + "</username>";
                xmlRequest += "	<reference>" + accountID + "</reference>";
                xmlRequest += "	<uprn>" + uprn + "</uprn>";
                xmlRequest += "	<memoText>" + memoText + "</memoText>";
                xmlRequest += "	<memoGroup>" + memoGroup + "</memoGroup>";

                xmlRequest += "</header>";
                xmlRequest += "<request>";
                xmlRequest += "<createnewmemo>";
                xmlRequest += "</createnewmemo>";
                xmlRequest += "</request>";
                xmlRequest += "</orbisrequest>";

                xmlResponse = orbisSend(xmlRequest);

            }
            catch (Exception ex)
            {
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }


            return xmlResponse;
        }




















        /// <summary>
        /// This API will return a list of all available document templates.
        /// </summary>
        /// <returns></returns>
        [WebMethod(Description = "Will return a list of all available document templates")]
        public string getDocumentTemplates()
        {

            string xmlResponse = string.Empty;
            string xmlRequest = string.Empty;

            try
            {

                xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                xmlRequest += "<header>";
                xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                xmlRequest += "	<type>" + this.getType + "</type>";
                xmlRequest += "</header>";
                xmlRequest += "<request>";
                xmlRequest += "<getdocumenttemplates>";
                xmlRequest += "</getdocumenttemplates>";
                xmlRequest += "</request>";
                xmlRequest += "</orbisrequest>";

                xmlResponse = orbisSend(xmlRequest);

            }
            catch (Exception ex)
            {
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }


            return xmlResponse;
        }




        /// <summary>
        /// This API will take in an account id and document type and attempt to create a document type against that account id. The result will be output.
        /// </summary>
        /// <returns></returns>
        [WebMethod(Description = "Will allow the user to request a document type to be created for a specific account id")]
        public string requestWordDoc(string accountID, string docType)
        {

            string xmlResponse = string.Empty;
            string xmlRequest = string.Empty;

            try
            {

                xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                xmlRequest += "<header>";
                xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                xmlRequest += "	<type>" + this.getType + "</type>";
                xmlRequest += "	<reference>" + accountID + "</reference>";
                xmlRequest += "	<doctype>" + docType + "</doctype>";
                xmlRequest += "</header>";
                xmlRequest += "<request>";
                xmlRequest += "<requestworddoc>";
                xmlRequest += "</requestworddoc>";
                xmlRequest += "</request>";
                xmlRequest += "</orbisrequest>";

                xmlResponse = orbisSend(xmlRequest);

            }
            catch (Exception ex)
            {
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }


            return xmlResponse;
        }






        /// <summary>
        /// This API will pass in an account id, discount code and discount start date and attempt to create that discount on the system. The result will be output.
        /// </summary>
        /// <returns></returns>
        [WebMethod(Description = "Will allow the user to create a discount of a specific type against an account")]
        public string createDiscount(string accountID, string discTypeCode, string discStartDate)
        {
            bool submit = true;
            string xmlResponse = string.Empty;
            string xmlRequest = string.Empty;
            string formattedStartDate = string.Empty;



            try
            {
                // Try and convert the string to a real date
                DateTime date = Convert.ToDateTime(discStartDate);

                string day = date.ToString("dd");
                string month = date.ToString("MMM");
                string year = date.ToString("yyyy");

                formattedStartDate = day + "/" + month + "/" + year;
                //throw new Exception("Day: " + day + ". Month: " + month + ". Year: " + year );

            }
            catch (Exception ex)
            {
                submit = false;
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }

            if (submit)
            {
                try
                {

                    xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                    xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                    xmlRequest += "<header>";
                    xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                    xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                    xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                    xmlRequest += "	<type>" + this.getType + "</type>";
                    xmlRequest += "	<reference>" + accountID + "</reference>";
                    xmlRequest += "	<disctypecode>" + discTypeCode + "</disctypecode>";
                    xmlRequest += "	<discstartdate>" + formattedStartDate + "</discstartdate>";
                    xmlRequest += "</header>";
                    xmlRequest += "<request>";
                    xmlRequest += "<creatediscount>";
                    xmlRequest += "</creatediscount>";
                    xmlRequest += "</request>";
                    xmlRequest += "</orbisrequest>";

                    xmlResponse = orbisSend(xmlRequest);

                }
                catch (Exception ex)
                {
                    xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
                }
            }

            return xmlResponse;
        }







        /// <summary>
        /// This API will pass in an account id, exemption code and exemption start date and attempt to create that exemption on the system. The result will be output.
        /// </summary>
        /// <returns></returns>
        [WebMethod(Description = "Will allow the user to create an exemption of a specific type against an account")]
        public string createPropExemption(string accountID, string exemptTypeCode, string exemptStartDate)
        {

            string xmlResponse = string.Empty;
            string xmlRequest = string.Empty;

            try
            {

                xmlRequest += "<?xml version='1.0' encoding='UTF-8' ?>";
                xmlRequest += "<orbisrequest xmlns='http://www.northgateps.com/xml/schemas/orbis'>";
                xmlRequest += "<header>";
                xmlRequest += "	<applicationid>" + this.getApplicationID + "</applicationid>";
                xmlRequest += "	<timestamp>" + this.getOracleTimeStamp + "</timestamp>";
                xmlRequest += "	<requestid>" + this.getRequestID + "</requestid>";
                xmlRequest += "	<type>" + this.getType + "</type>";
                xmlRequest += "	<reference>" + accountID + "</reference>";
                xmlRequest += "	<exempttypecode>" + exemptTypeCode + "</exempttypecode>";
                xmlRequest += "	<exemptstartdate>" + exemptStartDate + "</exemptstartdate>";
                xmlRequest += "</header>";
                xmlRequest += "<request>";
                xmlRequest += "<createpropexemption>";
                xmlRequest += "</createpropexemption>";
                xmlRequest += "</request>";
                xmlRequest += "</orbisrequest>";

                xmlResponse = orbisSend(xmlRequest);

            }
            catch (Exception ex)
            {
                xmlResponse = "<erc><error>" + ex.Message + "</error></erc>";
            }


            return xmlResponse;
        }





        #endregion




        /// <summary>
        /// Make the request to an Orbis Stored Procedure.
        /// The Stored Procedure expects a Clob input and returns a Clob and an Integer output
        /// </summary>
        /// <param name="xml"></param>
        /// <returns></returns>
        private string orbisSend(string xml)
        {


            string response = string.Empty;

			// Connect to ORBIS via the Oracle Client
			// Windows server will need Orace Net Manager installed and a connection established: https://docs.oracle.com/cd/B10501_01/network.920/a96580/admintoo.htm
			// Connection string is defined in the web.config file
            string cnn = ConfigurationManager.ConnectionStrings["ORBISConnectionString"].ConnectionString;

            try
            {
                using (OracleConnection conn = new OracleConnection(cnn))
                {
             
                    conn.Open();

                    OracleCommand cmd = new OracleCommand();
                    cmd.Connection = conn;
                    cmd.CommandText = "CTAXDBA.XML_API.MAIN";
                    cmd.CommandType = CommandType.StoredProcedure;


                    // The XML_API exects and CLOB input, a CLOB output and a number output
                    var inputParam1 = cmd.Parameters.Add("V_Xml_String", OracleDbType.Clob, xml, ParameterDirection.Input);
                    var outputParam1 = cmd.Parameters.Add("v_temp_clob_out", OracleDbType.Clob, ParameterDirection.Output);
                    var outputParam2 = cmd.Parameters.Add("v_return_status", OracleDbType.Int16, ParameterDirection.Output);

                    // Execute the command
                    cmd.ExecuteNonQuery();

                    // Get the Clob response
                    response = (outputParam1.Value as OracleClob).Value;


                }
            }
            catch (Exception ex)
            {
                response = "<erc><orbisSend><error>" + ex.Message + "</error></orbisSend></erc>";
            }


            return response;



        }


    }
}
